package SRules;  
#Erik Velldal, 2003, erikve@ilf.uio.no

use strict;


#to find required packages:
use FindBin; 
use lib $FindBin::Bin;
#or set
#use lib '/hf/omilia/site/home/erikve/spartan';

use SPatterns;
use SUtils qw(:IO :MATCH);

use locale;
use POSIX qw(locale_h ceil floor);
setlocale(LC_ALL, "norwegian"); 

use Exporter;
use vars qw/@ISA @EXPORT/;

@ISA = qw(Exporter);

@EXPORT = qw(

	     &noun_conj_noun
	     &verb_noun_obj             
	     &noun_subj_verb            
	     &verb_noun_subj            
	     &verb_noun_ind_obj         
	     &noun_subj_obj_sbu_rel_verb
	     &adj_noun_mod              
	     &noun_subj_pred            
	     &poss_noun_mod             
	     &noun_mod_noun             
	     &noun_prep_noun            
	     &verb_prep_obj 

    );









#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#                           RULES
#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=







#     ------------------------------------------------------
#                       Conjunctions
#     ------------------------------------------------------



sub noun_conj_noun {
    my $buf = shift;
    my ($nouns_1, $nouns_2, $con);
    my $con_pos = my $noun_1_pos = my $noun_2_pos  = 0;
    my $experience = 0;
    while(($con, $con_pos) = find_constituent($ENUM_CONJ_B_PAT, $buf, $con_pos)) {
	++$con_pos;
	if(($nouns_2,$noun_2_pos) = next_is_np($buf, $con_pos)
	   and ($nouns_1,$noun_1_pos) = (prev_constituent($NOUN_PAT, $buf, $con_pos))) {
	    print_rel($noun_1_pos, $noun_2_pos, $buf, "N_con_N", $nouns_1, $nouns_2);
	}
    }
}





#     ------------------------------------------------------
#                  Adjective-Noun Relations
#     ------------------------------------------------------



sub adj_noun_mod {
    my $buf = shift; 
    my ($adjectives, $nouns);
    my $adj_pos = my $noun_pos = 0;
    while(($adjectives, $adj_pos) = find_constituent($ADJ_MOD_PAT, $buf, $adj_pos)) {

	$adjectives = ['ORDO'] if look_ahead($ORDINAL_PAT, $buf, $adj_pos, ($adj_pos + 2));
	if (($nouns, $noun_pos) = find_constituent($NOUN_PAT, $buf, ++$adj_pos)) {
	    print_rel($adj_pos, $noun_pos, $buf, 'A_mod_N', $adjectives, $nouns);
	}
    }
}








#     ------------------------------------------------------
#                          Possesives
#     ------------------------------------------------------



sub poss_noun_mod {
    my $buf = shift; 
    my ($poss, $nouns);
    my $poss_pos = my $noun_pos = 0;
    while(($poss, $poss_pos) = find_constituent($DET_POSS_NOUN_PAT, $buf, $poss_pos)) {
	++$poss_pos;
	if (($nouns, $noun_pos) = next_is_np($buf, $poss_pos)) {
	    print_rel($poss_pos, $noun_pos, $buf, 'N_poss_N', $poss, $nouns);
	}
    }
}



#     ------------------------------------------------------
#            Modifiserende Substantiv:  @subst>
#     ------------------------------------------------------


sub noun_mod_noun {
    my $buf = shift; 
    my ($mods, $nouns);
    my $mod_pos = my $noun_pos = 0;
    while(($mods, $mod_pos) = find_constituent($NOUN_MOD_PAT, $buf, $mod_pos)) {
	++$mod_pos;
	if (($nouns, $noun_pos) = next_is_np($buf, $mod_pos)) {
	    print_rel($mod_pos, $noun_pos, $buf, 'N_mod_N', $mods, $nouns);
	}
    }
}



#     ------------------------------------------------------
#                    Verb - Object Relations
#     ------------------------------------------------------



#noise-testen er helt avhengig av at use locale; setlocale(LC_ALL,
#"norwegian"); er satt for at ikke verb med  etc skal bli rensket
#bort. 

sub any_noise {
    my $strs = shift;
    foreach $_ (@$strs) {
	return 1 if m/(?:[^\w]|[\d_])/o;
    }
}
#\s is excluded by way of ^\w
#_ is neccessary because it's inlcuded in \w


sub verb_noun_obj {
    my $buf = shift; 
    my ($verbs, $nouns);
    my $noun_pos = my $verb_pos = 0;
    my $verb_range_delim = qr/\s(?:prep|$S_GR|sbu|inf-merke|verb|$CLB)\s/;
    while(($verbs, $verb_pos) = find_constituent($VERB_NO_AUX_PAT, $buf, $verb_pos)) {
	$noun_pos = ++$verb_pos;
	while(($nouns, $noun_pos) = find_constituent($NOUN_OBJ_PAT, $buf, $noun_pos)) {           
	    if(! look_ahead($verb_range_delim, $buf, $verb_pos, $noun_pos)) {
		print_rel( $verb_pos, $noun_pos, $buf, 'V_O_N', $verbs, $nouns) unless any_noise($verbs); 	
		++$noun_pos;	    
	    }
	    else { last; }
	}
    }
}





sub noun_obj_verb {
    my $buf = shift; 
    my ($verbs, $nouns, $del);
    my $verb_pos = my $noun_pos = my $delim_pos = 0;
    my $verb_range_delim = qr/\s(?:$S_GR |sbu|inf-merke|$CLB)\s/; 
    while(($nouns, $noun_pos) = find_constituent($NOUN_OBJ_PAT, $buf, $noun_pos)) { #$unambig_noun_obj_pat
	$delim_pos = $verb_pos = ++$noun_pos;
#	if (! find_constituent($VERB_NO_AUX_PAT, $buf, 0, $n_pos)) {
	while(($verbs, $verb_pos) = find_constituent($VERB_NO_AUX_PAT, $buf, $verb_pos)) {
	    ($del, $delim_pos) = look_ahead($verb_range_delim, $buf, $delim_pos, $verb_pos);
	    if(! $del) {
		print_rel($verb_pos, $noun_pos, $buf, 'V_O_N', $verbs, $nouns)  unless any_noise($verbs);  
		$delim_pos = ++$verb_pos; 	    
	    }
	    else { last; } 
	}
    }
}
    






#     ------------------------------------------------------
#            Verb - Indirect Object Relations
#     ------------------------------------------------------




sub verb_noun_ind_obj {
    my $buf = shift; 
    my ($verbs, $nouns);
    my $verb_pos = my $noun_pos = 0;
    my @entries;
    my $entry;
    my $verb_range_delim = qr/\s(?:$I_OBJ|$S_GR |sbu|inf-merke|verb|$CLB)\s/;
    while(($nouns, $noun_pos) = find_constituent($NOUN_IND_OBJ_PAT, $buf, $noun_pos)) {           
	++$noun_pos;
	if((($verbs, $verb_pos) = rev_find_constituent($VERB_DITRANS_NO_AUX_PAT, $buf, $noun_pos)) &&
#	if((($verbs, $verb_pos) = rev_find_constituent($VERB_NO_AUX_PAT, $buf, $noun_pos)) &&
	   (! look_ahead($verb_range_delim, $buf, ++$verb_pos, $noun_pos))) {
		    print_rel($verb_pos, $noun_pos, $buf, 'V_IO_N', $verbs, $nouns) unless any_noise($verbs); 
		}
    }
}










#     ------------------------------------------------------
#                    Verb - Subject Relations
#     ------------------------------------------------------





sub noun_subj_verb {
    my $buf = shift; 
    my ($verbs, $nouns);
    my $noun_pos = my $verb_pos = my $verb_tmp_pos = 0;
    my $verb_range_delim = qr/\s(?:$SUBJ|$S_GR |sbu|$CLB)\s/;
    while(($verbs, $verb_pos) = find_constituent($VERB_NO_AUX_PAT, $buf, $verb_pos)) {
	$verb_tmp_pos = $verb_pos;
	if((($nouns, $noun_pos) = rev_find_constituent($NOUN_SUBJ_PAT, $buf, $verb_tmp_pos)) && 
	   (! look_ahead($verb_range_delim, $buf, ++$noun_pos, $verb_tmp_pos))) {	 
	    print_rel($verb_pos, $noun_pos, $buf, 'V_S_N', $verbs, $nouns)  unless any_noise($verbs); 
	}
	$verb_pos = ++$verb_tmp_pos;
    }
}



sub verb_noun_subj {
    my $buf = shift; 
    my ($verbs, $una_nouns, $amb_nouns);
    my $np_temp = my $noun_pos = my $verb_tmp_pos = my $verb_pos = 0;
    my $verb_range_delim = qr/\s(?:$S_GR |sbu|inf-merke|$CLB|verb|$KOMMA)\s/;
    while(($verbs, $verb_pos) = find_constituent($VERB_NO_INF_NO_AUX_PAT, $buf, $verb_pos)) {
	$verb_tmp_pos = $verb_pos;
	$noun_pos = ++$verb_tmp_pos;
	while((($una_nouns, $np_temp) = find_constituent($UNAMBIG_NOUN_SUBJ_PAT, $buf, $noun_pos)) or
	      (($amb_nouns, $np_temp) = find_constituent($NOUN_SUBJ_PAT, $buf, $noun_pos))) {           
	    $noun_pos = ++$np_temp;
	    if(! look_ahead($verb_range_delim, $buf, $verb_tmp_pos, $noun_pos)) {
		if ($una_nouns) {
		    print_rel($verb_pos, $noun_pos, $buf, 'V_S_N', $verbs, $una_nouns)  unless any_noise($verbs); 
		} elsif (prev_constituent($COMMA_PAT, $buf, $verb_tmp_pos)) {
		    print_rel($verb_pos, $noun_pos, $buf, 'V_S_N', $verbs, $amb_nouns)  unless any_noise($verbs); 
		} else { last; }
	    } else { last; }
	}
	$verb_pos = $verb_tmp_pos;}
}



#     ------------------------------------------------------
#       Verb-Object/Subject Relations in Relative Clauses
#     ------------------------------------------------------



sub noun_subj_obj_sbu_rel_verb {
    my $buf = shift; 
    my ($verbs, $nouns); 
    my $temp_b;
    my $verb_pos = my $noun_pos =  my $sbu_pos = my $temp = 0;
    while(($temp_b, $temp) = look_ahead($SBU_REL_B_PAT, $buf, $sbu_pos)) {	
	$sbu_pos = $temp;
	$verb_pos = ++$sbu_pos; 
	if ((($nouns, $temp) = rev_find_constituent($NOUN_PAT, $buf, $sbu_pos, $noun_pos))          
	    && (! look_ahead($PRON_B_PAT, $buf, $temp, $sbu_pos))) {
	    $noun_pos = $temp;
	    my $obj_flag = 1;  #ndvendig for  kunne nye oss med kun n while-lkke for begge modus
	    while(($verbs, $temp) = find_constituent($VERB_NO_AUX_PAT, $buf, $verb_pos)) {
		$verb_pos = $temp;
		if(($obj_flag) &&
		   (look_ahead($NOMINAL_SUBJ_B_PAT, $buf, $sbu_pos, $temp)) &&
		   (! look_ahead($SBU_REL_OR_SGR_B_PAT, $buf, $sbu_pos, $temp))) {
		    print_rel($verb_pos, $noun_pos, $buf, 'V_O_N', $verbs, $nouns)  unless any_noise($verbs);
		    $verb_pos = ++$temp;
		    $obj_flag = 1;
		} elsif (! look_ahead($SBU_REL_OR_SGR_OR_NOM_SUBJ, $buf, $sbu_pos, $temp)) {
		    print_rel($verb_pos, $noun_pos, $buf, 'V_S_N', $verbs, $nouns)  unless any_noise($verbs);
		    $verb_pos = ++$temp;
		    $obj_flag = 0;
		} else { last; }
	    }
	} 
	$sbu_pos = $verb_pos;
	++$noun_pos;
    }
}





#     ------------------------------------------------------
#                    Prepositional Objects
#     ------------------------------------------------------



sub verb_prep_obj {
    my $buf = shift; 
    my ($verbs, $preps, $nouns);
    my $verb_pos = my $prep_pos = my $noun_pos = 0;
#   while(($verbs, $verb_pos) = find_constituent($PART_VERB_NO_AUX_PAT, $buf, $verb_pos)) {
    while(($verbs, $verb_pos) = find_constituent($VERB_NO_AUX_PAT, $buf, $verb_pos)) {
	++$verb_pos;
	if (($preps, $prep_pos) = verb_attch_prep($buf, $verb_pos) and  
	    ($nouns, $noun_pos) = next_is_np($buf, ++$prep_pos)) {
	    print_quad_rel($verb_pos, $prep_pos, $noun_pos, $buf, "V_prep_N", $verbs,  $preps, $nouns) unless any_noise($verbs);
	}
    }
}











#     ------------------------------------------------------
#                       Preposisjoner
#     ------------------------------------------------------


 


sub noun_prep_noun {
    my $buf = shift;
    my ($preps, $head_nouns, $fill_nouns, $verbs);
    my $prep_pos =  my $head_pos = my $fill_pos = my $verb_pos = 0;
    while(($fill_nouns, $fill_pos) = find_constituent($NOUN_PREP_FILL_PAT, $buf, $fill_pos)) {
	++$fill_pos;
	if (($preps, $prep_pos) = rev_find_constituent($PREP_PAT, $buf, $fill_pos) and
	     ($head_nouns, $head_pos) = rev_find_constituent($NOUN_PAT, $buf, $prep_pos) and
	    ! (look_ahead('(?i:clb)|sbu|\@(?i:s-gr)|verb|pron', $buf, $head_pos, $prep_pos)) and
	    next_is_np($buf, ($prep_pos + 1))) {

	    if (($verbs, $verb_pos) = rev_find_constituent($VERB_C_PAT, $buf, $head_pos)) {

		if (! next_constituent($PART_ADV_VERB_PAT, $buf, $verb_pos) or
		    look_ahead('sbu|\@(?i:s-gr)|(?i:clb)|prep', $buf, $verb_pos, $head_pos)) {
		    print_quad_rel($head_pos, $prep_pos, $fill_pos, $buf, "N_prep_N", $head_nouns,  $preps, $fill_nouns);  
		}

	    }
	    else {
		print_quad_rel($head_pos, $prep_pos, $fill_pos, $buf, "N_prep_N", $head_nouns, $preps, $fill_nouns);  
	    }
	}
    }
}





#     ------------------------------------------------------
#                Subjekt-/ Objekt-Predikativ
#     ------------------------------------------------------


#Bten er faktisk ogs en bil. ==>
#N_pred_N        bil     bt     6       1       ?

# Vitsen om Bondevik var ikke sttende ==>
#A_mod_N stte   vits    6       1       7       ?

sub noun_subj_pred {
    my $buf = shift; 
    my $pred_type;
    my ($base, $pred, $verb, $sbu);
    my $base_pos = my $pred_pos = my $verb_pos = my $sbu_pos = 0;
    my $temp_b; 
    my $temp;

    while(($temp_b, $temp) =  look_ahead($SUBJ_PRED_C_PAT, $buf, $pred_pos)) {
	$pred_pos = $temp;
	#setter relasjonstype:

	if(($pred, $temp) = next_constituent($ADJ_SUBJ_PRED_PAT, $buf, $pred_pos)) {

	    $pred = ['ORDO'] if look_ahead($ORDINAL_PAT, $buf, $temp, ($temp + 2));
	    #"/bilen/ var den /frste/ av sitt slag"
	    $pred_type = "A_mod_N"; }

	elsif(($pred, $temp) = next_constituent($NOUN_SUBJ_PRED_PAT, $buf, $pred_pos)) { 
	    $pred_type = "N_pred_N"; }

#	elsif(($pred, $temp) = next_constituent($POSS_NOUN_SUBJ_PRED_PAT, $buf, $pred_pos)) { 
#	    $pred_type = "N_poss_N"; } 
#slr nesten aldri til, kommenteret ut til fordel for effektivitet. 


	else { $base_pos = ++$pred_pos; next; }
	#ignorerer sbu / prep / inf-merke / det / etc med @s-pred tagg forelpig	
	$base_pos = ++$pred_pos;

	if(($temp_b, $base_pos) = rev_find_constituent($SUBJ_C_PAT, $buf, $base_pos) and 
#	   ($base, $base_pos)  = next_is_np($buf, $temp, ($base_pos - 1))    and
	   #frste subjektsbase fr predikativet er et substantiv
	   ! look_ahead('\@(?i:s-gr)', $buf, ++$base_pos, $pred_pos) and
	   ($temp_b, $temp) = rev_find_constituent($VERB_C_PAT, $buf, $pred_pos) and
	   ($verb, $verb_pos) = next_constituent($PRED_VERB_PAT, $buf, $temp, ($pred_pos - 1)) and
	   ! look_ahead('<komma>', $buf, $verb_pos, $pred_pos)) {
	    #frste verb bakover fra subjekts-predikativet er predikativt
	    if(($base, $base_pos)  = next_is_np($buf, ($base_pos - 1), ($pred_pos - 1)) and 
	       ! look_ahead('\s\"som\"\ssbu\s', $buf, $base_pos, $pred_pos)) { 
		#slr til p alle sbu-former av "som".. virker ikke som @<sbu-rel var stabil nok..!?
		print_rel($pred_pos, $base_pos, $buf, $pred_type, $pred, $base);  # *1
	    }

	    elsif (($temp_b, $base_pos) = rev_find_constituent($NOMINAL_C_PAT, $buf, $pred_pos) and
		   ($temp, $sbu_pos) = find_constituent('\s\"som\"\ssbu\s', $buf, $base_pos, $pred_pos) and
		   #finn nrmeste mulige base fra predikativet, uavhengig av @subj-tagging
		   ($base_pos < $sbu_pos) and
		   #basen m st fr sbu'en.
		   #vil kun ha substantiver:
		   ($base, $base_pos) = next_is_np($buf, $base_pos)) {
		    print_rel($pred_pos, $base_pos, $buf, $pred_type, $pred, $base); # *2
	    }

	}

# *1 =  /vitsen/ om Bondevik var ikke /sttende/    A_mod_A
# *2 = !/vitsen/ om /Bondevik/, som har vrt /syk/, var ikke !/morsom/

#!/../ = ikke dekket 

    }
}




1;
