package SUtils;
#Erik Velldal, 2003, erikve@ilf.uio.no

use strict;
#require v5.5;

use diagnostics;

#to find required packages:
use FindBin; 
use lib $FindBin::Bin;
#or set
#use lib '/hf/omilia/site/home/erikve/spartan';

use SPatterns;

use locale;
use POSIX qw(locale_h ceil floor);
setlocale(LC_ALL, "norwegian"); 

#  print +(sort grep /\w/, map { chr } 0..255), "\n"; 
#  ==>
# _0123456789AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz



use Exporter;
use vars qw/@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS/;
@ISA = qw(Exporter);



@EXPORT = qw/

    &slurp_sentence 

    /;


@EXPORT_OK = qw/

    &print_sent_buf 
    &recover_tagged_text 
    &print_rel  
    &print_quad_rel  
    &print_sent_word_form
    &del_asterisks
 

    &rev_find_constituent   
    &find_constituent  
    &next_constituent  
    &look_ahead  
    &prev_constituent
    &next_is_np
    &verb_attch_prep
    &count_preceding_wordforms
    $S_ID

    /;





%EXPORT_TAGS = (IO => [qw( &slurp_sentence 
			   &print_sent_buf 
			   &recover_tagged_text 
			   &print_rel  
			   &print_quad_rel  
			   &print_sent_word_form 
			   &del_asterisks)], 


		MATCH => [qw( &rev_find_constituent   
			      &find_constituent  
			      &next_constituent  
			      &look_ahead
			      &prev_constituent
			      &next_is_np

			      &verb_attch_prep	
	
			      &count_preceding_wordforms
			      )]);






#global sentence id / counter:
our $S_ID = 0;




#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#                            I / O
#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=


sub del_asterisks { 
    foreach $_ (@_) {
	s/\*//g;
    }
}
#del_asterisks "*w*o*r*d" ==> "word"


sub print_rel {

    my ($pos1, $pos2, $buf, $rel, $elm1, $elm2) = @_;
    my ($e1, $e2);
    my $e1_id = count_preceding_wordforms($buf,$pos1);
    my $e2_id = count_preceding_wordforms($buf,$pos2);
     
    unless($OSLO_BERGEN_V) {
	del_asterisks(@$elm1); 
	del_asterisks(@$elm2); 
    }

    foreach $e1 (@$elm1) {
	foreach $e2 (@$elm2) {
	    print "$rel\t$e1\t$e2\t$e1_id\t$e2_id\t$S_ID\n";
	}
    }
}


sub print_quad_rel {
    my ($pos1, $pos2, $pos3, $buf, $rel, $elm1, $elm2, $elm3) = @_;     
    my $e1_id = count_preceding_wordforms($buf,$pos1);
    my $e2_id = count_preceding_wordforms($buf,$pos2);
    my $e3_id = count_preceding_wordforms($buf,$pos3);
    my ($e1, $e2, $e3);
    
    unless($OSLO_BERGEN_V) {
	del_asterisks(@$elm1); 
	del_asterisks(@$elm2); 
	del_asterisks(@$elm3); 
    }
     
    foreach $e1 (@$elm1) {
	foreach $e2 (@$elm2) {
	    foreach $e3 (@$elm3) {
		print "$rel\t$e1\t$e2\t$e3\t$e1_id\t$e2_id\t$e3_id\t$S_ID\n";
	    }
	}
    }
}
 


	
#print function for inspection / DBUG
sub print_sent_buf { 
    my $buf = shift;
    my $f_i = shift;
    for (my $i = 0; $i < $#{@$buf}; $i++) {
	unless($i == $f_i) { print "$$buf[$i]"; }
    }
}



sub recover_tagged_text {
    my $filepath = shift;
    my $s_buf;
    open(IN, $filepath) or die "drainbamage: $!\n";
    while($s_buf = slurp_sentence(\*IN)) {
	print_sent_word_form($s_buf);
    }
    close(IN);
}


sub print_sent_word_form {
    my $buf = shift;
    my $pos = 0; 
    while($pos < $#{@$buf}) {
	if ($$buf[$pos] =~ m/$WORD_FORM_PAT/) {
	    print "$1 ";
	}
	    ++$pos;
    }
    print "\n\n";
}





#slurp_sentence takes a filehandle argument and returns next sentence
#as delimited by global variable $SENTENCE_DELIM. in the case og EOF,
#undef is returned.
sub slurp_sentence { 
    my $FH = shift;
    my @buff = ();
    my $line;
    do {
	$line = <$FH>;
	if (! $line) {return;}
	push(@buff,$line);
    } until ($line =~ m/$SENTENCE_DELIM/); 
    return \@buff;
}












#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#                Pattern Matching and Parsing
#     =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=









sub rewind_to_wordform {
    my ($buf, $pos, $end) = @_;
    $end ||= 0;
    until(($pos <= $end) || ($$buf[$pos] =~ m/$WORD_FORM_B_PAT/)) { --$pos; }
    $pos;
}


sub forward_to_wordform {
    my ($buf, $pos, $end) = @_;
    $end ||= $#{@$buf};
    until(($pos >= $end) || ($$buf[$pos] =~ m/$WORD_FORM_B_PAT/)) { ++$pos; }
    $pos;
}


sub count_preceding_wordforms {
    my ($buf, $pos) = @_;
    my $point = $pos; 
    my $count = 0;
    
    #just in case..;
    if($point >= $#{@$buf}){
	$point = --$#{@$buf};}
    elsif($point < 0){
	$point = 0;}
    
    until($point < 0) {
	if($$buf[$point] =~ m/$WORD_FORM_B_PAT/) 
	{ ++$count; }
	--$point;
    }
    $count;
}


sub purge_entry { 
    my ($buf, $pos, $pat) = @_;
    $pat ||= $LEMMA;
    my %lemma_tab;
    until(($pos >= $#{@$buf}) ||   #supposedly impossible but..
	  ($$buf[$pos] =~ m/$WORD_FORM_B_PAT/)) {
	if($$buf[$pos] =~ m/$pat/) {	    
	    $lemma_tab{$1}++ if defined($1);;
	}
	++$pos;
    }
    return keys(%lemma_tab);
}



 
sub verb_attch_prep {
    my ($buf, $pos) = @_; 
    (($pos) || ($pos = 0)); 
    my $end =  $#{@$buf};
    my @lemma;
    $pos = forward_to_wordform($buf, $pos, $end);
    while(++$pos != $end) {

	$_ = $$buf[$pos];

	next if /$WORD_FORM_B_PAT/;
	last if /$PREP_PAT/ && (@lemma = ($1));
	next if /$ADV/; #i.e. adjectival adverb (preposisitonal check allready done above).
	last; 

    }
    if(@lemma) {
	until($$buf[--$pos] =~ $WORD_FORM_B_PAT) { }
	return (\@lemma, $pos); 
    } else {return;} #= the empty list, when called from a list/array context
} #position =  buffer position of word-form pertaining to the block of the found element. 







sub next_is_np {
    my ($buf, $pos) = @_; 
    (($pos) || ($pos = 0)); 
    my $end =  $#{@$buf}; #$end = end pos in buffer, or length if not provided.
    my @lemma_list;
#fast forward to fresh wordform # 1
    $pos = forward_to_wordform($buf, $pos, $end); 
#   until(($pos >= $end) || ($$buf[$pos] =~ m/$WORD_FORM_B_PAT/)) { ++$pos; }
    while(++$pos != $end) {

	$_ = $$buf[$pos];

#$VERB_RANGE_DELIM_PAT1 = qr/\s(?:prep|$S_GR|sbu|inf-merke|verb|$CLB)\s/;

	next if /$WORD_FORM_B_PAT/;
	next if /\sadj\s/;
#	next if /$ADJ_MOD_PAT/;
	next if /$DET_MOD_PAT/;
	next if /\sadv\s/;            #m sjekkes nrmere
#bruker 'adv' fremfor @adv for  ikke slippe gjennom preposisjoner etc..
	next if /$SUBST_MOD_PAT/; #sjelden, ikke verdt overheaden?!?  viktigst for navn.
#	next if /$COMMA_PAT/; #sjelden, ikke verdt overheaden?!?
#hvis komma slippes igjennom br man ha et flagg for nr komma er ok; f.eks hvis vi allerede har sett et adjektiv.
       
	if(/$NOUN_PAT/){
	    push @lemma_list, $1;
	    until($$buf[++$pos] =~ m/$WORD_FORM_B_PAT/) {
		if(($$buf[$pos] =~ m/$NOUN_PAT/) && 
		   (! (grep {/$1/} @lemma_list))) { 
		    push @lemma_list, $1; }
	    } last; 
	}
	else { last; } 

    }
    if(@lemma_list) {
	until($$buf[--$pos] =~ m/$WORD_FORM_B_PAT/) { }
	return (\@lemma_list, $pos); 
    } else {return;} #= the empty list, when called from a list/array context
} #position =  buffer position of word-form pertaining to the block of the found element. 






#prev_constituent brukes av bla verb_noun_subj noun_conj_noun:


sub prev_constituent { 
    my ($pat, $buf, $pos, $end) = @_; 
    #(pattern to search for, sentence array reference, start position in array, search end position)
    $end ||= 0; #$end = end position in array (length of referenced array if no position is specified)
    my $wf_pos;
    my @lemma_list;

    $pos = rewind_to_wordform($buf, $pos, $end);
    --$pos;

    while($pos > $end) { #check if previous element match the pattern.
	if($$buf[$pos] =~ m/$WORD_FORM_B_PAT/) { last; }  #exit on new wordform	    
	if($$buf[$pos] =~ m/$pat/) { 

	    $wf_pos = $pos = rewind_to_wordform($buf, $pos, $end);
	    @lemma_list = purge_entry($buf, ++$pos, $pat);
	    last; }

	else {--$pos;}
    }
    if (@lemma_list) {
	return (\@lemma_list, $wf_pos);} #return lemma and buffer position
    else {return;} #= the empty list, when called from a list/array context
}





sub rev_find_constituent {
    my ($pat, $buf, $pos, $end) = @_; 
    ($end ||=  0); #$end = end pos in buffer, or length if not provided.
    my @lemma_list;
    my $wf_pos;
    $pos = rewind_to_wordform($buf, $pos, $end);
    while($pos-- != $end) { #search for next element matching pattern
	if($$buf[$pos] =~ m/$pat/) {

	    $wf_pos = $pos = rewind_to_wordform($buf, $pos, $end);
	    @lemma_list = purge_entry($buf, ++$pos, $pat);

	    last;
	}
    }
    if (@lemma_list) {
	return (\@lemma_list, $wf_pos);} #return lemma and buffer position
    else {return;} #= the empty list, when called from a list/array context
}






#find_constituent(pattern, sentence-array, start-position, ?end-position?)

sub find_constituent {
    my ($pat, $buf, $pos, $end) = @_; 
    $pos ||= 0; 
    $end ||=  $#{@$buf}; #$end = end pos in buffer, or length if not provided.
    my @lemma_list; 

    #fast forward to fresh wordform:
    $pos = forward_to_wordform($buf, $pos, $end);

    while($pos++ != $end) {
	if($$buf[$pos] =~ m/$pat/) {
	    @lemma_list = purge_entry($buf, $pos, $pat);
	    last;
	}
    }
#    if(@lemma_list) {  #  annen sjekk for retur, ellers false ved treff p non-capturing pat.
    if($pos < $end ) {  #  E.V: 2003-01-29
	$pos = rewind_to_wordform($buf, --$pos);
	return (\@lemma_list, $pos); #return list of lemmas and buffer position of word form
    } else {return;} #= the empty list, when called from a list/array context
} #position =  buffer position of word-form pertaining to the block of the found element.  



sub find_constituent_DBG {
    my ($pat, $buf, $pos, $end) = @_; 
    $pos ||= 0; 
    $end ||=  $#{@$buf}; #$end = end pos in buffer, or length if not provided.
    my @lemma_list; 

    #fast forward to fresh wordform:
    $pos = forward_to_wordform($buf, $pos, $end);

    while($pos++ != $end) {
	if($$buf[$pos] =~ m/$pat/) {
	    @lemma_list = purge_entry($buf, $pos, $pat);
	    print "it's a hit: \$pos $pos \$end $end\n";
	    last;
	}
    }
    print "out of while: \$pos $pos \$end $end\n";
    if(@lemma_list) {
	
	$pos = rewind_to_wordform($buf, --$pos);
	return (\@lemma_list, $pos); #return list of lemmas and buffer position of word form
    } else {return;} #= the empty list, when called from a list/array context
} #position =  buffer position of word-form pertaining to the block of the found element.  








sub next_constituent {
    my ($pat, $buf, $pos) = @_; 
    (($pos) || ($pos = 0)); 
    my $end =  $#{@$buf}; #$end = end pos in buffer, or length if not provided.
    my @lemma_list;
#fast forward to fresh wordform # 1                                                    

    my $wf_pos = $pos = forward_to_wordform($buf, $pos, $end);

    while(++$pos != $end) {
	if($$buf[$pos] =~ m/$pat/){

	    @lemma_list = purge_entry($buf, $pos, $pat);
	    last;
	}
	elsif($$buf[$pos] =~ m/$WORD_FORM_B_PAT/) { last; }
    }
    if(@lemma_list) {
	return (\@lemma_list, $wf_pos); #return list of lemmas and buffer position of word form
    } else {return;} #= the empty list, when called from a list/array context
} #position =  buffer position of word-form pertaining to the block of the found element. 







#($sbu_pos = look_ahead($SBU_REL_B_PAT, $buf, $sbu_pos))

#returns two values if search i successfull; true and the position of the hit.
#otherwise false (as depending on context of call). rationale; the
#position might be 0,  which could be interpreted as false by
#the recieving context. 


sub look_ahead { 
    my ($pat, $buf, $pos, $end) = @_; 
    $end ||= $#{@$buf};  #$end = end pos in buffer, or length if not provided.    
    $pos = forward_to_wordform($buf, $pos, $end);
    
    while($pos++ != $end) {
	last if $$buf[$pos] =~ m/$pat/;
   }
    if ($pos < $end) {
	$pos = rewind_to_wordform($buf, --$pos);
#	until($$buf[--$pos] =~ $WORD_FORM_B_PAT) { }
	return wantarray ? (1, $pos) : 1;  # = (TRUE, buffer position)
    } else {return;}
}
#'wantarray': when return of "anonymous arrray", avoids ',' interpreted as scalar comma operator.






1;











