#!/usr/bin/perl -w

#Erik Velldal, 2003, erikve@ilf.uio.no

# transforms dependency triplets and quadruples (as produced by tagparse.pl) 
# into features of individual words ('local contexts'). 
#V-S-N drive car w1_id w2_id s_id ==> SUBJ_OF drive car w1_id w2_id s_id 
#V-prep-N give up hope w1_id w2_id s_id ==> PREP_up_OBJ_OF give hope w1_id w2_id s_id
#reads files with suffix ".tap" and outputs to (and creates) files with ".loc"-extension
#(.tap = tag parse  .loc = local context)
#
#switches: 
# -W = leave out word id's
# -S = leave out sentence id 
# -t = print to standard out

use strict;
use diagnostics;


#to find required packages:
use FindBin; 
use lib $FindBin::Bin;
#or set
#use lib '/hf/omilia/site/home/erikve/spartan';

use SPatterns qw($LETTER);

use File::Basename;
use locale;
use POSIX qw(locale_h ceil floor);
setlocale(LC_ALL, "norwegian"); 



use Getopt::Std;
our $opt_t = 0; #print to STDOUT
our $opt_S = 0; #suppress sentence id
our $opt_W = 0; #suppress word id
getopts('SWt');





our $OH = *STDOUT; #output fileshandle




sub prepare_strings {
    foreach $_ (@_) {
	s/\s/_/g;	 #replace white space with _
	s/(?:\d+)/NUM/g; #replace numerical units with NUM
    }
}




sub trim_print_feat {
    my ($rel, $w1, $w2, $w1_id, $w2_id, $s_id) = @_;
#    $w1 =~ s/\s/_/g;
#    $w2 =~ s/\s/_/g;

    #trim:
    foreach $_ ($w1, $w2) {
	s/\s/_/g;	 #replace white space with _
	s/(?:\d+)/NUM/g; #replace numerical units with NUM    
    }

#    prepare_strings($w1, $w2);

    #print:
    print $OH "$rel\t$w1\t$w2";
    print $OH "\t$w1_id\t$w2_id" if (! $opt_W);
    print $OH "\t$s_id" if (! $opt_S);
    print $OH "\n";   
}



open(LOG,"> r2f.log") || die "$!: r2f.log";


  
while (@ARGV) {
    my $file = shift;
    if (-d $file) {
	opendir(DIR, $file) || die "drainbamage: $! : $_\n";
	my $dir_name = $file;
	my @files = map { "$dir_name/$_" } grep !/(^\.|\#)|\#$|~$/, readdir(DIR);
	closedir DIR || die "$!\n";
	unshift(@ARGV, @files);
 	next; 
    }
    
    my ($base,$path,$syn_suf) = &File::Basename::fileparse($file, '\.tap'); 
    next if (! $syn_suf); 

    unless($opt_t) {
	$path .= $base . '.loc';
	open(OUT,"> $path") || die "$!: $path";
	$OH = *OUT;
    }
    open(IN, $file) || die "$!: $file";



 
#    my $word_pair_w_id =  qr/\t($LETTER+)\t($LETTER+)\t(\d+)$/;
#    my $word_trip_w_id = qr/\t($LETTER*)\t($LETTER*)\t($LETTER*)\t(\d+)$/;

    my $word_pair_w_id =  qr/\t($LETTER+)\t($LETTER+)\t(\d+)\t(\d+)\t(\d+)$/;
    my $word_trip_w_id = qr/\t($LETTER*)\t($LETTER*)\t($LETTER*)\t(\d+)\t(\d+)\t(\d+)\t(\d+)$/;

   
    while(<IN>) {
	
      SWITCH: {

	  /^  V_O_N     $word_pair_w_id /x &&  
	      do { trim_print_feat('OBJ_OF', $1, $2, $3, $4, $5);         last SWITCH; };    

	  /^  V_S_N     $word_pair_w_id /x && 
	      do { trim_print_feat('SUBJ_OF', $1, $2, $3, $4, $5);        last SWITCH; };    

	  /^  V_IO_N    $word_pair_w_id /x && 
	      do { trim_print_feat('IND_OBJ_OF', $1, $2, $3, $4, $5);     last SWITCH; }; 

	  /^  N_prep_N  $word_trip_w_id /x &&                  

#             inkluderer preposisjonen i relasjonsnavnet:
#	      do { trim_print_feat("PP_$2_MOD_OF", $1, $3, $4, $6, $7);   
#		   trim_print_feat("PP_$2_MOD_BY", $3, $1, $6, $4, $7);   last SWITCH; }; 

	      do { trim_print_feat("PP_MOD_OF", $1, $3, $4, $6, $7);   
		   trim_print_feat("PP_MOD_BY", $3, $1, $6, $4, $7);   last SWITCH; }; 
	  
	  /^  A_mod_N   $word_pair_w_id /x && 
	      do { trim_print_feat('ADJ_MOD_BY', $1, $2, $3, $4, $5);     last SWITCH; }; 

#	  /^  V_prep_N  $word_trip_w_id /x && 
#	      do { trim_print_feat("PREP_$2_OBJ_OF", $1, $3, $4, $6, $7); last SWITCH; };

	  /^  V_prep_N  $word_trip_w_id /x && 
	      do { trim_print_feat("PREP_OBJ_OF", "$1_$2", $3, $4, $6, $7); last SWITCH; };

	  /^  N_con_N   $word_pair_w_id /x &&  
	      do { trim_print_feat('NOUN_CON', $1, $2, $3, $4, $5);
		   trim_print_feat('NOUN_CON', $2, $1, $4, $3,  $5);
#		   trim_print_feat('NOUN_CON', $1, $1, $3, $3, $5);
#		   trim_print_feat('NOUN_CON', $2, $2, $4, $4, $5);       
		   last SWITCH; }; 
		                                   
	  /^  N_poss_N  $word_pair_w_id /x &&                   
	      do { trim_print_feat('POSS_OF', $2, $1, $4, $3, $5);     
		   trim_print_feat('POSS_BY', $1, $2, $3, $4, $5);     
		                                         last SWITCH; }; 
	  /^  N_mod_N   $word_pair_w_id /x &&                    
	      do { trim_print_feat('NOUN_MOD_OF', $2, $1, $4, $3, $5);
		   trim_print_feat('NOUN_MOD_BY', $1, $2, $3, $4, $5);    last SWITCH; }; 
	
#	  /^  N_pred_N  $word_pair_w_id /x && 
#	      do { trim_print_feat('HAS_PRED_FILL', $1, $2, $3);
#		   trim_print_feat('PRED_FILL_OF', $2, $1, $3);   last SWITCH; }; 
	  
	  print LOG "Unrecognized relation: $_ in $file\n\n";   last SWITCH; };
    }
}




#V_prep_N        snakke  til forsamlingen  ==> 
#PREP_OBJ_OF     snakke_til forsamlingen 



#N_prep_N        glasset p  bordet  ==>  
#PP_p_MOD_OF    glasset     bordet 
#PP_p_MOD_BY    bordet      glasset



#N_poss_N        nasjonens   tilstand  ==>             
#POSS_OF         tilstand    nasjonen
#POSS_BY         nasjonen    tilstand 



#N_mod_N         glass       vin       ==>
#NOUN_MOD_OF     vin         glass
#NOUN_MOD_BY     glass       vin



#A_mod_N         populistisk politiker    ==>
#ADJ_MOD_BY      populistisk politiker


#N_con_N         l          vin       ==>
#NOUN_CON        l          vin
#NOUN_CON        vin         l
#NOUN_CON        l          l
#NOUN_CON        vin         vin








