#!/usr/bin/perl -w

#Erik Velldal, 2003, erikve@ilf.uio.no

#Parses tagged files with .sy or .syn-extension and extracts dependency
#triplet/quadruples with corresponding sentence ids. Output for
#"some_file.syn" is written to "some_file.tap", or to terminal/STDOUT if
#switch -t is set.   
#-o = oslo-version. no -o means oslo-bergen version. 
#-i = initiate sentence counter, default is 0 (nb: this means first
#sentence's id = 1). useful when working with a corpus that is spread
#out over several files that are processed separately.

#output format (tab separated):
#dep-rel  word1  word2  (word3?)  word1-id  word2-id  (word3-id?)  sentence-id



use diagnostics;
use strict;
#require v5.5;


#to find required packages:
use FindBin; 
use lib $FindBin::Bin;
#or set
#use lib '/hf/omilia/site/home/erikve/spartan';

use SRules;
use SUtils qw($S_ID &slurp_sentence &print_sent_word_form);
use SPatterns qw($OSLO_BERGEN_V $SENTENCE_DELIM);


use File::Basename;
use locale;
use POSIX qw(locale_h ceil floor);
setlocale(LC_ALL, "norwegian"); 





use Getopt::Std;
my %opts;
getopt('i', \%opts);



$SUtils::S_ID = ($opts{i} || 0);


$OSLO_BERGEN_V = ($opts{o}) ? 0 : 1; 
unless($OSLO_BERGEN_V) { $SENTENCE_DELIM = qr/CLB.*?\@\?\?\?/; }

#print "$SPatterns::OSLO_BERGEN_V\n";




while (@ARGV) {
    my $file = shift; #next file or dir argument
    if (-d $file) { #dips down in found directory arg:
	opendir(DIR, $file) || die "drainbamage: $! : $_\n";
	my $dir_name = $file; #saves away path, to glue it back on later.
	my @files = map { "$dir_name/$_" } grep !/(^\.|\#)|\#$|~$/, readdir(DIR);
	#glues the directory path onto files. sifts away files of form:  .foo and foo~ and #foo#
	closedir DIR || die "$!\n";
	unshift(@ARGV, @files);
 	next;  #prepends the files in the directory to the (possibly empty) list of remaining files.
    }


    my ($base,$path,$syn_suf) = &File::Basename::fileparse($file, ('\.syn', '\.sy'));
    next if (! $syn_suf);
    
    unless($opts{t}) { #unless "print to terminal/STDOUT"-switch set
	$path .= $base . '.tap';
	open(STDOUT,"> $path") || die "$!: $path";
    }



    relex($file);

}





sub relex {
     
    my $filepath;
    my $buffer;
    foreach $filepath (@_) {
	open(IN,$filepath) or die "drainbamage: $!\n";
	while($buffer = slurp_sentence(\*IN)) {
	    ++$SUtils::S_ID; #increment sentence id counter 

#	    print_sent_word_form($buffer); #DBUG
#	    print "$SUtils::S_ID\n"; #DBUG

	    noun_conj_noun($buffer);	         #N_con_N    
	    verb_prep_obj($buffer);              #V-PART

	    verb_noun_obj($buffer);              #FV-OBJ / IV-OBJ
	    noun_subj_verb($buffer);             #SUBJ-FV / SUBJ-IV
	    verb_noun_subj($buffer);             #FV-SUBJ / IV-SUBJ
            verb_noun_ind_obj($buffer);          #FV/IV-IND_OBJ
	    noun_subj_obj_sbu_rel_verb($buffer); #relativized S-V / O-V   

	    adj_noun_mod($buffer);               #ADJ-NOUN
#	    noun_subj_pred($buffer);             #SUBJ_PRED-NOUN #works fine, but rare and expensive..
	    poss_noun_mod($buffer);              #POSS-NOUN
	    noun_mod_noun($buffer);              #@subst>
	    noun_prep_noun($buffer);             #N_prep_N

## noun_obj_verb(\@buffer);              #OBJ-FV / OBJ-IV  #too noisy..


	}
	
	close(IN) or die $!;
    }
}

